@@ -1,27 +1,55 @@
-// Copyright (C) 2006-2007 W. Pantke <gnome-color-chooser@punk-ass-bitch.org>
-//  
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// 
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+/* GNOME Color Chooser - GTK+/GNOME desktop appearance customization tool
+ * Copyright (C) 2006-2009 Werner Pantke <wpantke@punk-ass-bitch.org>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Project email: <gnome-color-chooser@punk-ass-bitch.org>
+ *
+ */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
-#include "configloader.h"
 #include "mainwindow.h"
+#include "treehandler.h"
+#include "configloader.h"
+#include "enginewindow.h"
 #include "infowindow.h"
+#include "validatorwindow.h"
 #include "gtkrcexporter.h"
 #include "utils.h"
+#include "exception.h"
+
+#include "colorbutton.h"
+#include "checkbutton.h"
+#include "spinbutton.h"
+#include "fontbutton.h"
+#include "filechooserbutton.h"
+#include "combobox.h"
+
+#include <glib/gi18n.h>
+#include <gtkmm.h>
 #include <iostream>
 #include <fstream>
 
+using std::cerr;
+using std::cout;
+using std::endl;
+using std::fstream;
+using std::ifstream;
+using std::ios;
 using namespace GnomeCC;
 
 
@@ -41,10 +69,10 @@ MainWindow::MainWindow(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   //set_border_width(10); // Sets the border width of the window.
   //stick(); // Window appears on all user desktops
 
+  m_refGlade->connect_clicked("main_revert",        sigc::mem_fun(*this, &MainWindow::on_main_revert));
   m_refGlade->connect_clicked("main_cancel",        sigc::mem_fun(*this, &MainWindow::on_main_cancel));
   m_refGlade->connect_clicked("main_apply",         sigc::mem_fun(*this, &MainWindow::on_main_apply));
-  //m_refGlade->connect_clicked("main_ok",            sigc::mem_fun(*this, &MainWindow::on_main_ok));
-  
+
   m_refGlade->connect_clicked("file_exit",          sigc::mem_fun(*this, &MainWindow::on_file_exit));
   m_refGlade->connect_clicked("file_new",           sigc::mem_fun(*this, &MainWindow::on_file_new));
   m_refGlade->connect_clicked("file_open",          sigc::mem_fun(*this, &MainWindow::on_file_open));
@@ -53,7 +81,9 @@ MainWindow::MainWindow(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   //m_refGlade->connect_clicked("file_export_as_caption",       sigc::mem_fun(*this, &MainWindow::on_file_export));
   m_refGlade->connect_clicked("file_export_as_gtp", sigc::mem_fun(*this, &MainWindow::on_file_export_as_gtp));
   m_refGlade->connect_clicked("file_install",       sigc::mem_fun(*this, &MainWindow::on_file_install));
-  
+
+  m_refGlade->connect_clicked("tools_validate",     sigc::mem_fun(*this, &MainWindow::on_tools_validate));
+
   m_refGlade->connect_clicked("help_gtkinfo",       sigc::mem_fun(*this, &MainWindow::on_help_gtkinfo));
   m_refGlade->connect_clicked("help_info",          sigc::mem_fun(*this, &MainWindow::on_help_info));
   
@@ -138,13 +168,16 @@ MainWindow::~MainWindow()
 
 void MainWindow::close()
 {
-	hide();
+  hide();
 }
 
 
 // connect and initialize all found widgets
 void MainWindow::init(TreeHandler* config,
                       string       configfile,
+                      string       filename,
+                      string       gtkrcfile,
+                      string       customgtkrcfile,
                       string       version,
                       string       icon,
                       string       image_path,
@@ -153,10 +186,14 @@ void MainWindow::init(TreeHandler* config,
 {
   this->setConfig(config);
   this->setVersion(version);
-  this->image_path   = image_path;
-  this->m_filename   = configfile;
-  this->m_configfile = configfile;
-  set_title((string(PACKAGE_NAME) + " " + version).c_str());
+  this->image_path        = image_path;
+  this->m_configfile      = configfile;
+  this->m_filename        = filename;
+  this->m_gtkrcfile       = gtkrcfile;
+  this->m_customgtkrcfile = customgtkrcfile;
+
+  set_title(PACKAGE_NAME);
+
   if(Utils::Io::check_file(icon))
   {
     this->m_refIcon = Gdk::Pixbuf::create_from_file(icon, 48, 48, true);
@@ -284,8 +321,8 @@ void MainWindow::init(TreeHandler* config,
   if(!ConfigLoader::load_profiles(profile_dir))
     cout << "Couldn't find any gnomecc profile!" << endl;
 
-  if(!ConfigLoader::load_config(m_configfile, m_pConfig, m_refGlade, this))
-    cout << "Couldn't load gnomecc theme!" << endl;
+  if(!ConfigLoader::load_config(m_filename, m_pConfig, m_refGlade, this))
+    cout << "Couldn't load gnomecc theme '" << m_filename << "'!" << endl;
 
   m_refGlade->get_widget_derived("profilebox", combobox);
   if(combobox)
@@ -295,7 +332,6 @@ void MainWindow::init(TreeHandler* config,
     combobox->init(m_pConfig);
   }
 
-
 }
 
 
@@ -505,32 +541,66 @@ void MainWindow::on_main_cancel()
 
 void MainWindow::on_main_apply()
 {
-  // save settings
-//  m_pConfig->save();
   if(m_configfile != "")
   {
-    ConfigLoader::export_config(m_pConfig, m_configfile, true, true); 
+
+    // check/create include statement in ~/.gtkrc-2.0
+    if(!Utils::check_include(
+          m_gtkrcfile,
+          "include \".gtkrc-2.0-gnome-color-chooser\"",
+          ".gtkrc-2.0-gnome-color-chooser"))
+    {
+
+      if(! (Utils::Io::check_file(m_gtkrcfile, true)
+            || Utils::Io::create_file(m_gtkrcfile)) )
+      {
+        g_warning(_("Could not open or create file %s."), m_gtkrcfile.c_str());
+        return;
+      }
+
+
+      if(!Utils::create_include(
+            m_gtkrcfile,
+            "include \".gtkrc-2.0-gnome-color-chooser\"",
+            ".gtkrc-2.0-gnome-color-chooser"))
+      {
+        g_warning(_("Unable to modify file %s."), m_gtkrcfile.c_str());
+        return;
+      }
+    }
+
+    // save settings
+    ConfigLoader::export_config(m_pConfig, m_configfile, true, true);
 
     // let all gtk apps redraw (save changes and let the apps re-read them)
-    this->write_gtkrc();
+    this->write_gtkrc(m_customgtkrcfile);
     this->redraw_all_gtk_apps();
+
   }
 }
 
 
-void MainWindow::on_main_ok()
+void MainWindow::on_main_revert()
 {
-//  m_pConfig->save(); // save database
+  m_pConfig->reset_to_defaults();
+//todo: create a (ConfigLoader::)reset_engine_combos();
+  ModWidget::reload_all_widgets();
 
-  if(m_configfile != "")
+  // remove_include statement in ~/.gtkrc-2.0
+  if(!Utils::remove_include(
+        m_gtkrcfile,
+        "include \".gtkrc-2.0-gnome-color-chooser\"",
+        ".gtkrc-2.0-gnome-color-chooser"))
   {
-    ConfigLoader::export_config(m_pConfig, m_configfile, true, true);
-    
-    // let all gtk apps redraw (save changes and let the apps re-read them)
-    this->write_gtkrc();
-    this->redraw_all_gtk_apps();
+    g_warning(_("Unable to modify file %s."), m_gtkrcfile.c_str());
   }
-  this->close();       // close this window
+
+  // save settings
+  ConfigLoader::export_config(m_pConfig, m_configfile, true, true);
+
+  // let all gtk apps redraw (save changes and let the apps re-read them)
+  this->write_gtkrc(m_customgtkrcfile);
+  this->redraw_all_gtk_apps();
 }
 
 
@@ -545,6 +615,7 @@ void MainWindow::on_file_new()
   m_pConfig->reset_to_defaults();
 //todo: create a (ConfigLoader::)reset_engine_combos();
   ModWidget::reload_all_widgets();
+
   m_filename = "";
 }
 
@@ -737,9 +808,7 @@ void MainWindow::on_dialog_save(int response_id)
         }
 
       }
-      else
-        cerr << "Invalid filename \"" << file << "\"" << endl;
-      
+
     }
     else
       dialog->hide();
@@ -754,6 +823,18 @@ void MainWindow::on_file_install()
 }
 
 
+void MainWindow::on_tools_validate()
+{
+  ValidatorWindow *pWindow = 0;
+  m_refGlade->get_widget_derived("validatorwindow", pWindow);
+  if(pWindow)
+  {
+    pWindow->init(this);
+    pWindow->show();
+  }
+}
+
+
 void MainWindow::on_help_gtkinfo()
 {
   InfoWindow* pWindow = 0;
@@ -778,9 +859,9 @@ void MainWindow::on_help_info()
   {
     pDialog->set_transient_for(*this);
     pDialog->set_name(PACKAGE_NAME);
-    pDialog->set_copyright("(c) 2006-2007 by Werner Pantke");
+    pDialog->set_copyright("Copyright (C) 2006-2009 Werner Pantke");
     pDialog->set_version(version.c_str());
-    pDialog->set_comments(_("A tool for customizing the appereance of the GNOME desktop"));
+    pDialog->set_comments(_("A tool for customizing the appearance of the GNOME desktop"));
     pDialog->set_logo(m_refIcon);
     pDialog->show();
   }
@@ -814,11 +895,9 @@ void MainWindow::redraw_all_gtk_apps()
 }
 
 
-void MainWindow::write_gtkrc()
+void MainWindow::write_gtkrc(string filename)
 {
-//todo: parameter: file to write to
-//  GtkrcExporter exporter = new GtkrcExporter();
-  string filename = getenv("HOME") + string("/.gtkrc-2.0-gnome-color-chooser");
+  g_return_if_fail(filename != "");
 
   Exporter* exporter = Exporter::create_exporter(Exporter::Gtkrc);
   try
@@ -827,7 +906,11 @@ void MainWindow::write_gtkrc()
   }
   catch(Exception &e)
   {
-    cerr << "Export to " << filename << " failed: " << e.get_string() << endl;
+
+    g_warning(
+          _("Export to file %s failed: %s."),
+          filename.c_str(),
+          e.get_string().c_str() );
   }
 
   delete exporter;
