@@ -1,23 +1,29 @@
-// Copyright (C) 2006-2007 W. Pantke <gnome-color-chooser@punk-ass-bitch.org>
-//  
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// 
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+/* GNOME Color Chooser - GTK+/GNOME desktop appearance customization tool
+ * Copyright (C) 2006-2008 Werner Pantke <wpantke@punk-ass-bitch.org>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Project email: <gnome-color-chooser@punk-ass-bitch.org>
+ *
+ */
 
 #include "treehandler.h"
 #include "utils.h"
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include <cstring>
+#include <iostream>
 
 using namespace GnomeCC;
 
@@ -231,7 +237,7 @@ void TreeHandler::unsetProfile()
   xmlNode* profile;
 
   // if someone has manually added multiple profiles, remove all of them  
-  while(profile = get_profile())
+  while( (profile = get_profile()) )
   {
     //cout << "profile " << getProfileName() << " removed" << endl;
     xmlUnlinkNode(profile);
@@ -244,22 +250,19 @@ void TreeHandler::unsetProfile()
 const string TreeHandler::getProfile()
 {
   xmlNode* profile;
-  const char* content;
   
-  if(profile = get_profile())
+  if( (profile = get_profile()) )
     return Utils::Xml::get_content(profile, "content");
       
   return "";
-
 }
 
 
 const string TreeHandler::getProfileName()
 {
   xmlNode* profile;
-  const char* property;
   
-  if(profile = get_profile())
+  if( (profile = get_profile()) )
     return Utils::Xml::get_property(profile, "name");
 
   return "";
@@ -347,7 +350,7 @@ const string TreeHandler::getCategoryShortDescription()
 
 const string TreeHandler::getCategoryLongName()
 {
-  return Utils::Xml::get_property(cat_cur, "long_name");
+  return Utils::Xml::get_content(cat_cur, "long_name");
 }
 
 
@@ -683,7 +686,7 @@ const string TreeHandler::getElementShortDescription()
 }
 
 
-void TreeHandler::setElementValue(char* value)
+void TreeHandler::setElementValue(const char* value)
 {
   if(el_cur != NULL)
   {
