@@ -1,33 +1,57 @@
-// Copyright (C) 2006-2007 W. Pantke <gnome-color-chooser@punk-ass-bitch.org>
-//  
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// 
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software 
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+/* GNOME Color Chooser - GTK+/GNOME desktop appearance customization tool
+ * Copyright (C) 2006-2009 Werner Pantke <wpantke@punk-ass-bitch.org>
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Project email: <gnome-color-chooser@punk-ass-bitch.org>
+ *
+ */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "utils.h"
 
+#include <fstream>
+#include <glib.h>
+#include <cstring>
+#include <unistd.h> // unlink()
+#include <stdio.h>  // rename()
+
 // Utils::Io
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h> // not platform independent :(
 
+// Utils::UI
+#include <gtkmm/dialog.h>
+#include <gtkmm/main.h>
+#include <gtkmm/messagedialog.h>
+#include <gtkmm/window.h>
 
+using std::fstream;
+using std::ofstream;
+using std::ifstream;
+using std::ios;
+using std::ios_base;
 using namespace GnomeCC;
 
 
 bool Utils::check_include(string filename, string include, string searchpattern)
 {
-  if(!Utils::Io::check_file(filename, true))
+  if(!Utils::Io::check_file(filename))
     return 0;
     
   fstream fi(filename.c_str(), ios_base::in);
@@ -37,12 +61,23 @@ bool Utils::check_include(string filename, string include, string searchpattern)
 
   //scan
   string line;
+  size_t p_include;
+  size_t p_pattern;
+  size_t p_comment;
 
   while ( getline ( fi, line ) )
   {
-    //string::size_type i = line.find_first_not_of ( " \t\n\v" );
+    p_include = string::npos;
+    p_pattern = string::npos;
+    p_comment = string::npos;
 
-    if ( line == include || (searchpattern != "" && line.find(searchpattern, 0) != string::npos) )
+    if ( line == include \
+          || (searchpattern != "" \
+                && ( p_pattern = line.find(searchpattern, 0) ) != string::npos \
+                && ( p_include = line.find("include", 0) ) != string::npos \
+                && p_include < p_pattern \
+                && ( ( p_comment = line.find('#',0) ) == string::npos \
+                      || p_comment > p_pattern) ) )
     {
       fi.close();
       return 1;
@@ -68,8 +103,68 @@ bool Utils::create_include(string filename, string include, string searchpattern
 
     gtkrc << include << "\n";
     gtkrc.close();
+    return 1;
   }
 
+  return 0;
+}
+
+
+// returns true if there is no matching include statement in the end,
+// no matter if there was one at all
+bool Utils::remove_include(string filename, string include, string searchpattern)
+{
+  if(!Utils::check_include(filename, include, searchpattern))
+    return 1;
+
+  string filename_tmp = filename + ".tmp";
+  fstream fi(filename.c_str(), ios_base::in);
+  fstream fo(filename_tmp.c_str(), ios_base::out | ios_base::trunc);
+
+  if(!fi || !fo)
+    return 0;
+
+  //scan
+  string line;
+  size_t p_include;
+  size_t p_pattern;
+  size_t p_comment;
+
+  while ( getline ( fi, line ) )
+  {
+    p_include = string::npos;
+    p_pattern = string::npos;
+    p_comment = string::npos;
+
+    if ( line == include \
+          || (searchpattern != "" \
+                && ( p_pattern = line.find(searchpattern, 0) ) != string::npos \
+                && ( p_include = line.find("include", 0) ) != string::npos \
+                && p_include < p_pattern \
+                && ( ( p_comment = line.find('#',0) ) == string::npos \
+                      || p_comment > p_pattern) ) )
+    {
+      // do nothing => remove this line
+    }
+    else
+      fo << line << "\n";
+  }
+  fi.close();
+  fo.close();
+
+  if(unlink(filename.c_str()))
+  {
+    g_debug("Could not delete file %s.", filename.c_str());
+    return 0;
+  }
+
+  if(rename(filename_tmp.c_str(), filename.c_str()))
+  {
+    g_debug("Could not rename file %s to %s.", filename_tmp.c_str(), filename.c_str());
+    return 0;
+  }
+
+  return 1;
 }
 
 
@@ -132,46 +227,91 @@ const string Utils::Xml::get_property(xmlNode* node, string property)
 }
 
 
-const string Utils::Xml::get_content(xmlNode* node, string content)
+
+// returns the most proper translation of a child content
+//   (using all locales a user has set).
+const string Utils::Xml::get_content(xmlNode* parent, string content)
+{
+  if(parent == NULL || content == "")
+    return "";
+
+  int      lang_score  = INT_MAX;
+  xmlNode *node        = NULL;
+  xmlNode *node_best   = NULL;
+  bool use_translation = Utils::Xml::needs_translation(content);
+
+  char const * const *langs = g_get_language_names ();
+
+
+  for(node = get_node(parent->children, content);
+        node != NULL;
+        node = get_node(node->next, content))
+  {
+    string lang = lang2string(node);
+
+    if(!use_translation && lang == "")
+      return content2string(node);
+
+    if(use_translation && lang != "")
+    {
+      for (int i = 0; langs[i] != NULL && i < lang_score; i++)
+      {
+        if(lang == string(langs[i]))
+        {
+          node_best  = node;
+          lang_score = i;
+        }
+      }
+    }
+    else if(lang == "" && node_best == NULL)
+      node_best = node;
+
+    if(lang_score == 0)
+      return content2string(node_best);
+
+  }
+  
+  return content2string(node_best);
+}
+
+
+const string Utils::Xml::content2string(xmlNode *node)
 {
 
-  if(node && content != "")
+  if(node)
   {
-    xmlNode *child = NULL;
-    char *loc = NULL;
-    string locale;
-
-    if(Utils::Xml::needs_translation(content) \
-          && (loc = setlocale(LC_ALL, NULL)) \
-          && strcmp(loc, "en_US")
-      )
+    xmlChar* content = xmlNodeGetContent(node);
+    if(content != NULL && strcmp((char*)content, ""))
     {
-      string output;
-      locale = loc;
-
-      if((output = get_lang(get_node(node->children, content), content, locale.c_str())) != "")
-        return output;
-
-      if(locale.find("@", 0) != string::npos)
-        if((output = get_lang(get_node(node->children, content), content, locale.substr(0, locale.find("@", 0)).c_str() )) != "")
-          return output;
-
-      if(locale.find(".", 0) != string::npos)
-        if((output = get_lang(get_node(node->children, content), content, locale.substr(0, locale.find(".", 0)).c_str() )) != "")
-          return output;
-
-      if(locale.find("_", 0) != string::npos)
-        if((output = get_lang(get_node(node->children, content), content, locale.substr(0, locale.find("_", 0)).c_str() )) != "")
-          return output;
+      const string temp = string((char*)content);
+      xmlFree(content);
+      return temp;
     }
+    xmlFree(content);
+  }
 
-    return get_lang(get_node(node->children, content), content, NULL, true);
+  return "";
+}
+
+const string Utils::Xml::lang2string(xmlNode *node)
+{
+  if(node)
+  {
+    xmlChar *lang = xmlNodeGetLang(node);
+    if(lang)
+    {
+      const string temp = string((char*)lang);
+      xmlFree(lang);
+      return temp;
+    }
+    xmlFree(lang); // super cleanex 2.0
   }
 
   return "";
 }
 
 
+/*
 const string Utils::Xml::get_lang(xmlNode* node, string content, const char* locale, bool acceptSpace)
 {
   xmlNode *child = node;
@@ -202,6 +342,10 @@ const string Utils::Xml::get_lang(xmlNode* node, string content, const char* loc
 
   return "";
 }
+*/
+
+
+
 
 
 xmlNode* Utils::Xml::get_node(xmlNode* pNode, string name)
@@ -222,6 +366,15 @@ xmlNode* Utils::Xml::get_node(xmlNode* pNode, string name)
 
 
 
+const string Utils::Xml::get_language(xmlNode *node)
+{
+  return lang2string(node);
+}
+
+
+
+
+
 // Utils::Xml privates
 const bool Utils::Xml::needs_translation(string content)
 {
@@ -287,7 +440,9 @@ bool Utils::Io::check_directory(string directory)
 {
   struct stat config_stat;
 
-  if(directory != "" && !stat( directory.c_str(), &config_stat) && (config_stat.st_mode & S_IFDIR))
+  if(directory != "" \
+        && !stat( directory.c_str(), &config_stat) \
+        && (config_stat.st_mode & S_IFDIR))
     return 1;
   
   return 0;
@@ -344,7 +499,7 @@ bool Utils::Io::remove_directory(string directory)
     else
     {
     
-      while(entry = readdir(dir))
+      while( (entry = readdir(dir)) )
       {
       
         string name = string(entry->d_name);
@@ -413,14 +568,13 @@ bool Utils::Io::copy_directory(string src, string dst)
 
   if(Utils::Io::check_directory(src))
   {
-
     DIR *dir = opendir(src.c_str());
     if(!dir)
       return 0;
     else
     {
     
-      while(entry = readdir(dir))
+      while( (entry = readdir(dir)) )
       {
       
         string name = string(entry->d_name);
@@ -454,7 +608,6 @@ bool Utils::Io::copy_directory(string src, string dst)
   }
   else
     return 0;
-  
 }
 
 
@@ -493,6 +646,100 @@ bool Utils::Io::copy_file(string src, string dst)
 
 
 
+
+
+
+
+
+
+
+// Utils::String
+
+
+string Utils::String::to_lower(string str)
+{
+  for (unsigned int i=0;i<strlen(str.c_str());i++)
+    if (str[i] >= 0x41 && str[i] <= 0x5A)
+      str[i] = str[i] + 0x20;
+
+  return str;
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+// Utils::Ui
+
+int Utils::Ui::m_quit_message_count = 0;
+
+Gtk::Window *Utils::Ui::m_dialog_parent = NULL;
+
+
+void Utils::Ui::set_dialog_parent_window(Gtk::Window *parent)
+{
+  m_dialog_parent = parent;
+}
+
+
+void Utils::Ui::print_error(
+      string message1,
+      string message2,
+      bool quit_mainloop)
+{
+  g_return_if_fail(message1 != "");
+  
+  Gtk::MessageDialog *dialog = new Gtk::MessageDialog(message1, false, Gtk::MESSAGE_ERROR);
+  dialog->set_modal(true);
+  dialog->set_title(PACKAGE_NAME);
+
+  if(message2 != "")
+    dialog->set_secondary_text(message2);
+
+  if(m_dialog_parent)
+    dialog->set_transient_for(*m_dialog_parent);
+
+  dialog->signal_response().connect(
+        sigc::bind( sigc::bind( sigc::ptr_fun(&Utils::Ui::close_dialog),
+              quit_mainloop), (Gtk::Dialog*)dialog) );
+
+  if(quit_mainloop)
+    m_quit_message_count++;
+
+  dialog->show();
+}
+
+
+void Utils::Ui::close_dialog(
+      int response_id,
+      Gtk::Dialog *dialog,
+      bool quit_mainloop)
+{
+  g_return_if_fail(dialog != NULL);
+
+  delete dialog;
+
+  if(quit_mainloop)
+  {
+    m_quit_message_count--;
+    
+    if(m_quit_message_count <= 0)
+      Gtk::Main::quit();
+  }
+}
+
+
+
+
+
 ///////////////////
 
 
@@ -857,11 +1104,3 @@ string Utils::gtkinfo()
 
 
 
-
-
-
-
-
-
-
-
